#!/cygdrive/c/Python27/python.exe
from PIL import Image, ImageDraw, ImageFont
import sys
from math import sqrt, pow
def write_colors(file_name, image):
    colors = []
    if (image.palette):
        # Convert to RGB from paletted image if necessary.
        image = image.convert('RGB') 
    f_txt = open("%s.txt" % file_name, "w")
    color_list = image.getcolors()
    index = 0
    f_txt.write("#\tcount\tRGB\n")
    for color_item in color_list:
        count, color = color_item
        colors.append([index, count, color])
        f_txt.write("%d\t%d\t[%d, %d, %d]\n" % (index, count, color[0], color[1], color[2]))
        index = index + 1
    f_txt.close()        
    return colors


def dist(color1, color2):
    """Calculate the distance between 2 colors"""
    return sqrt(
        pow(color1[0]-color2[0],2) + 
        pow(color1[1]-color2[1],2) + 
        pow(color1[2]-color2[2],2))
    
    
def draw_palette(file_name, color_list, pixel_width):
    num_colors = len(color_list)
    # Create swatches for the palette.  Make the swatches square, and make it fit
    rows  = int(sqrt(num_colors))
    cols = rows
    pixels= pixel_width/rows
    p = Image.new('RGB', (pixel_width, pixel_width))
    draw = ImageDraw.Draw(p)
    x = 0
    y = 0
    row = 0
    col = 0
    font = ImageFont.truetype("arial.ttf", 50)
    white = (255, 255, 255)
    black = (0, 0, 0)
    for index, count, color in color_list:
        x1 = col*pixels+0
        y1 = row*pixels+0
        x2 = col*pixels+pixels
        y2 = row*pixels+pixels
        if (dist(white, color) < dist(black, color)):
            textcolor = black
        else:
            textcolor = white
        draw.rectangle( [x1, y1, x2, y2], fill = color)
        
        draw.text([x1, y1], "%d, %d" % (index, count), font = font, fill=textcolor)
        # Also, create an individual jpg for each palette color.  This can be cheaper to print your swatches.
        sx = 1000
        sy = 800
        swatch = Image.new('RGB', (sx, sy))
        swatch_draw = ImageDraw.Draw(swatch)
        swatch_draw.rectangle( [0, 0, sx, sy], fill = color)
        swatch_draw.text([sx/2, sy/2], "%d, %d" % (index, count), font = font, fill=textcolor)
        del swatch_draw
        swatch.save("%s.%d.jpg" % (file_name, index), quality=95)
        col = (col + 1)
        if (col == cols):
            row = row+1
            col = 0
    del draw
    p.save(file_name, quality=95)
        

def create_color_has(color_list):
    color_hash = {}
    for index, count, color in color_list:
        color_name = "%d_%d_%d" % ( color[0], color[1], color[2])
        color_hash[color_name] = index
    return color_hash

def draw_final_image(output_file, input_image, color_list, pixel_size_in_pixels = 100):
    # Draw the image input_image with big pixels, and marked with which color index it belongs to.
    input_image = input_image.convert('RGB')
    color_hash = create_color_has(color_list) # Make reverse color lookup easy
    
    cols, rows = input_image.size

    output_image = Image.new('RGB', ((cols+2)*pixel_size_in_pixels, 
                                     (rows+2)*pixel_size_in_pixels))
    draw = ImageDraw.Draw(output_image)
    font = ImageFont.truetype("arial.ttf", 50)
    white = (255, 255, 255)
    black = (0, 0, 0)
    for col in range(cols):
        for row in range(rows):
            color = input_image.getpixel((col, row))
            color_name = "%d_%d_%d" % ( color[0], color[1], color[2])
            if (dist(white, color) < dist(black, color)):
                textcolor = black
            else:
                textcolor = white
            x1 = (col+1) * pixel_size_in_pixels
            y1 = (row+1) * pixel_size_in_pixels
            x2 = x1 + pixel_size_in_pixels
            y2 = y1 + pixel_size_in_pixels
            draw.rectangle( [x1, y1, x2, y2], fill = color)
            draw.text([x1, y1], "%d" % color_hash[color_name], font = font, fill=textcolor)

    for row in (-1, rows):
        for col in range(cols):
            x1 = (col+1) * pixel_size_in_pixels
            y1 = (row+1) * pixel_size_in_pixels
            x2 = x1 + pixel_size_in_pixels
            y2 = y1 + pixel_size_in_pixels
            draw.rectangle( [x1, y1, x2, y2], fill = white)
            draw.text([x1, y1], "%d" % col, font = font, fill=black)
    for col in (-1, cols):
        for row in range(rows):
            x1 = (col+1) * pixel_size_in_pixels
            y1 = (row+1) * pixel_size_in_pixels
            x2 = x1 + pixel_size_in_pixels
            y2 = y1 + pixel_size_in_pixels
            draw.rectangle( [x1, y1, x2, y2], fill = white)
            draw.text([x1, y1], "%d" % row, font = font, fill=black)
    del draw
    output_image.save(output_file, quality=95)

if (len(sys.argv) < 2):
    print "You must specify a file name for your low-resolution, reduced color image.  For example:\ncreate_quilt.exe my_flower.bmp\n"

for name in sys.argv[1:]:
    im = Image.open(name)
    pixels = im.size[0]*im.size[1]
    print "There are %d colors in this image, and %d pixels: " % (len(im.getcolors(pixels)), pixels)
    color_list = write_colors("color_list", im)
    draw_palette("palette.jpg", color_list, 2000)
    draw_final_image(output_file = "quilt.jpg", 
                     input_image = im, 
                     color_list  = color_list, 
                     pixel_size_in_pixels = 100)
    
        
